/******************************************************************************
* Include files
******************************************************************************/
#include <avr/io.h>
#include <avr/interrupt.h>
#include "timer.h"

/******************************************************************************
* Macros
******************************************************************************/

#define FrqMeasStart 0
#define FrqWeating 1

#define Timer3Enable  TCCR3B = TCCR3B | (1 << CS30)
#define Timer3Disable TCCR3B = TCCR3B & ~(1 << CS30)
#define InputCapture3IT_Enable TIMSK3 = TIMSK3 | (1<<ICIE3)
#define InputCapture3IT_Disable TIMSK3 = TIMSK3 & ~(1<<ICIE3)

/******************************************************************************
* Constants
******************************************************************************/

/******************************************************************************
* Global Variables
******************************************************************************/
uint16_t Cnt=0;
volatile uint8_t Task_1000ms;
volatile uint8_t IC3count;
volatile uint8_t NL1;
volatile uint8_t NH1;
volatile uint8_t NL2;
volatile uint8_t NH2;
volatile uint8_t FrequMeasReady=FrqMeasStart;
uint16_t frequx;
uint8_t FrqMeasState;

/******************************************************************************
* External Variables
******************************************************************************/

/******************************************************************************
* Local Function Declarations
******************************************************************************/

/******************************************************************************
* Local Function Definitions
******************************************************************************/

/******************************************************************************
* Global Function Definitions
******************************************************************************/

/******************************************************************************
* Function:         void TIMER0_Init(void)
* Description:		Configure Timer0 to generate external signal for testing
* Input:
* Output:
* Notes:
******************************************************************************/
void TIMER0_Init(void)
{
	//use TIMER0
	//        D7    D6    D5     D4    D3    D2   D1   D0
	//TCCR0A: FOC0A WGM00 COM0A1 CM0A0 WGM01 CS02 CS01 CS00
	//        0     0     0      0     1     1    0    1
	//CTC mode, div=1024 (7812.5 Hz)
	
	TCCR0A = (1 << WGM01) | (1 << CS02) | (1 << CS00);
	OCR0A = 3;  //f = 8MHz/(1024*4) = 1953,1 Hz
	TIMSK0 = TIMSK0 | (1 << OCIE0A); // output compare IT enabled
}

/******************************************************************************
* Function:         void TIMER0_Init(void)
* Description:		Configure Timer0 to generate external signal for testing
* Input:
* Output:
* Notes:
******************************************************************************/
void TIMER2_Init(void)
{
	//use TIMER2
	//        D7    D6    D5     D4    D3    D2   D1   D0
	//TCCR2A: FOC2A WGM20 COM2A1 CM2A0 WGM21 CS22 CS21 CS20
	//        0     0     0      0     1     1    0    1
	//CTC mode, div=1024 (7812.5 Hz)
	
	TCCR2A = (1 << WGM21) | (1 << CS22) | (1<<CS21) | (1 << CS20);
	OCR2A = 77;  //f = 8MHz/(1024*78) = 100,16 Hz
	TIMSK2 = TIMSK2 | (1 << OCIE2A); // output compare IT enabled
}

/******************************************************************************
* Function:         void TIMER3_Init(void)
* Description:		Configure Timer3 to measure the time between two rising edges
* Input:
* Output:
* Notes:
******************************************************************************/
void TIMER3_Init(void)
{
	//          D7     D6     D5     D4     D3     D2     D1   D0
	//TCCR3A: COM3A1 COM3A0 COM3B1 COM3B0 COM3C1 COM3C0 WGM31 WGM30
	//           0      0      0      0      0     0      0     0
	// Outputs disconnected

	//          D7     D6     D5     D4     D3     D2     D1   D0
	//TCCR3B: ICNC3   ICES3    -    WGM33  WGM32   CS32  CS31 CS30
	//           0      1      0      0      0     0      0     1
	//normal mode, no prescaling, capture on rising edge
	//Macros to start and stop the timers

	TCCR3A = 0x00; //Outputs disconnected
	TCCR3B = TCCR3B | (1 << ICES3); //Capture on rising edge
	Timer3Disable;
}

/******************************************************************************
* Function:         void StartFrequMeasuring(void)
* Description:		Starting of freq. measuring
* Input:
* Output:
* Notes:
******************************************************************************/
void StartFrequMeasuring(void)
{
	Timer3Disable;
	TCNT3H = 0x00;
	TCNT3L = 0x00;
	Timer3Enable;

	IC3count = 0;
	FrequMeasReady = 0;
	InputCapture3IT_Enable;
}

/******************************************************************************
* Function:         void FrequMeas(void)
* Description:		Measuring time between two rising edges and calulating freq.
* Input:
* Output:
* Notes:
******************************************************************************/
void FrequMeas(void)
{
	unsigned int N1;
	unsigned int N2;

	switch(FrqMeasState)
	{
		case FrqMeasStart:
		StartFrequMeasuring();
		FrqMeasState = FrqWeating;
		break;
		case FrqWeating:
		if(FrequMeasReady)
		{
			N1 = (NH1 << 8) + NL1;
			N2 = (NH2 << 8) + NL2;
			frequx = 8000000UL/(N2-N1);
			FrqMeasState = FrqMeasStart;
		}
		break;
		default:FrqMeasState = FrqMeasStart;
		break;
	}
}

/******************************************************************************
* Interrupt Routines
******************************************************************************/

/******************************************************************************
* Function:         ISR(TIMER0_COMP_vect)
* Description:		Timer0 CTC IT
* Input:
* Output:
* Notes:
******************************************************************************/
ISR(TIMER0_COMP_vect)
{
	PORTA = PORTA ^ fx_mask; // square-wave f=976.6Hz
}

/******************************************************************************
* Function:         ISR(TIMER2_COMP_vect)
* Description:		Timer2 CTC IT
* Input:
* Output:
* Notes:
******************************************************************************/
ISR(TIMER2_COMP_vect)
{
	Cnt++;
	if (0 == Cnt % 100)
	  Task_1000ms = TRUE;
}

/******************************************************************************
* Function:         ISR(TIMER3_CAPT_vect)
* Description:      Timer3 input capture IT
* Input:
* Output:
* Notes:
******************************************************************************/
ISR(TIMER3_CAPT_vect)
{
	
	IC3count++;
	switch(IC3count)
	{
		case 1:	NL1 = ICR3L;
		NH1 = ICR3H;
		break;
		case 2:	NL2 = ICR3L;
		NH2 = ICR3H;
		FrequMeasReady = 1;
		InputCapture3IT_Disable;
		break;
	}
}

